use std::collections::HashMap;

use mongod::{assert_reference_of, Reference, Validate};

#[derive(Debug, Clone, Serialize, Deserialize, Model, Referencable)]
pub struct FlowInfo {
    pub _id: String,
    pub name: String,
    pub depends: HashMap<String, i64>,
    pub next: Option<String>,
    pub produces: Option<Vec<String>>,
}

impl Validate for FlowInfo {
    async fn validate(&self) -> Result<(), String> {
        Ok(())
    }
}

impl FlowInfo {
    pub async fn add(&mut self, id: &str) {
        self._id = id.to_string();
        self.insert_overwrite().await.unwrap();
    }
}

impl ToAPI for FlowInfo {
    async fn api(&self) -> serde_json::Value {
        json!({
            "id": self._id,
            "name": self.name,
            "depends": self.depends,
            "next": self.next,
            "produces": self.produces
        })
    }
}

use mongod::{
    derive::{Model, Referencable},
    Model, Referencable, ToAPI,
};
use mongodb::bson::doc;
use serde::{Deserialize, Serialize};
use serde_json::json;

/// A production flow
#[derive(Debug, Clone, Serialize, Deserialize, Model, Referencable)]
pub struct Flow {
    /// ID
    pub _id: String,
    /// Tiemstamp when the flow was started
    pub started: i64,
    /// Timestamp when the flow was ended
    pub ended: i64,
    /// Kind of flow; ID of the describing JSON
    pub kind: String,
    /// The flow succedding this one
    pub next: Option<Reference>,
}

impl Validate for Flow {
    async fn validate(&self) -> Result<(), String> {
        if let Some(next) = &self.next {
            assert_reference_of!(next, Flow);
        }

        Ok(())
    }
}
